/****************************************************************************
** common.h - contains all the common variables and functions for Psi
** Copyright (C) 2001, 2002  Justin Karneges
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#ifndef COMMON_H
#define COMMON_H

#include<qstring.h>
#include<qpixmap.h>
#include<qimage.h>
#include<qstringlist.h>
#include<qwidget.h>
#include<qmessagebox.h>
#include<qprocess.h>
#include"message.h"
#include"varlist.h"
#include"iconset.h"
#include"kmp/client.h"
#include"kmp/types.h"


#define PROXY_NONE      0
#define PROXY_HTTPS     1
#define PROXY_SOCKS4    2
#define PROXY_SOCKS5    3

#define STATUS_OFFLINE  0
#define STATUS_ONLINE   1
#define STATUS_AWAY     2
#define STATUS_XA       3
#define STATUS_DND      4

// global
struct PsiGlobal
{
	QString pathBase, pathHome, pathProfiles;
};

extern QString activeProfile;

extern QStringList dtcp_hostList;
extern int dtcp_port;
extern bool dtcp_use;

// options stuff
enum { cOnline, cOffline, cAway, cDND, cGroupFore, cGroupBack, cListBack };
enum { fRoster, fMessage, fChat };
enum { eMessage, eChat1, eChat2, eSystem, eOnline, eOffline, eSend };
enum { dcClose, dcHour, dcDay, dcNever };

struct Options
{
	QColor color[7];
	QString font[3];
	QString iconset;
	int alertStyle;

	bool useleft, singleclick, askOnline, popupMsgs, popupChats, raise;
	bool alwaysOnTop, noAwaySound, noAwayPopup, rosterAnim;
	bool useDock, dockDCstyle, dockHideMW, dockToolMW;
	int delChats, browser;

	int defaultAction;
	int incomingAs;
	VarList sp;

	int asAway, asXa, asOffline;
	bool use_asAway, use_asXa, use_asOffline;
	QString asMessage;
	QString onevent[7];

	// Added by Kiko 020621: points to the directory where the trusted
	// certificates used in validating the server's certificate are kept
	QString trustCertStoreDir;

	QString player;
	QString customBrowser, customMailer;

	bool ignoreHeadline, scrollTo, keepSizes, useEmoticons, alertOpenChats, chatSays;

	QSize sizeEventDlg, sizeChatDlg;
	bool jidComplete, grabUrls, noGCSound;
};

// functions

QString getResourcesDir();
QString getHomeDir();

QString getHistoryDir();

QString qstrlower(QString);
int qstrcasecmp(const QString &str1, const QString &str2);
int qstringlistmatch(QStringList &list, const QString &str);
QString qstringlistlookup(QStringList &list, int x);
QString CAP(const QString &str);

QString status2txt(int status);

QString eatptag(QString txt);
QString qstrquote(const QString &, int width=60, bool quoteEmpty=FALSE);
QString plain2rich(const QString &);
QString rich2plain(const QString &);
QString clipStatus(const QString &str, int width, int height);
QString expandEntities(const QString &in);
QString resolveEntities(const QString &);
QString linkify(const QString &);
QString emoticonify(const QString &in);

QString encodePassword(const QString &, const QString &);
QString decodePassword(const QString &, const QString &);

const QString & transport2str(const QString &name);
QPixmap & transport2icon(const QString &, int status);

QPixmap *loadImage(const QString &fname, const QStringList &dirs);
char *loadImageData(const QString &fname, const QStringList &dirs, int *size);

void bringToFront(QWidget *w);
QString jidEncode(const QString &jid);
QString jidDecode(const QString &jid);

QString jidnick(const QString &jid, const QString &nick);

int hexChar2int(char c);
char int2hexChar(int x);

QString logencode(QString);
QString logdecode(const QString &);

void unfinishedSoftware();

void openURL(const QString &);

QString getOSName();
int getTZOffset();
QString getTZString();

bool fileCopy(const QString &src, const QString &dest);

#ifdef Q_WS_MAC
void stopDockTileBounce();
void bounceDockTile();
#endif

void soundPlay(const QString &);

Jabber::Status makeStatus(int, const QString &);
int makeSTATUS(const Jabber::Status &);

void replaceWidget(QWidget *, QWidget *);
void closeDialogs(QWidget *);


struct TransportIconSet
{
	QPixmap *p[5];
};

extern TransportIconSet tic[5];

extern
QString PROG_NAME, PROG_VERSION;

extern
QPixmap *pix_blank,
	*pix_logo,
	*pix_icon_48,
	*pix_smile,
	*pix_find;

extern char *pixdat_ft_back, *pixdat_ft_file, *pixdat_ft_folder;
extern int pixlen_ft_back, pixlen_ft_file, pixlen_ft_folder;

extern
Jabber::Client *debug_jabber;

extern
Options option;

extern
PsiGlobal g;

extern
PsiIconSet is;

extern
bool useSound;

#endif
